/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_CACHE_H__
#define __PASIS_CACHE_H__

#include "pasis_config.h"
#include "pasis_debug.h"

#include "pasis_lists.h"
#include "pasis_freelist.h"

#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif

typedef struct pasis_cache_obj_s {
    unsigned int hash;
    unsigned int pin_count;
    PASIS_Q_PTRS (struct pasis_cache_obj_s) chain;
    PASIS_Q_PTRS (struct pasis_cache_obj_s) lru_q;
} pasis_cache_obj_t;

typedef PASIS_Q_STRUCT(pasis_cache_queue_s, pasis_cache_obj_t)
     pasis_cache_queue_t;

typedef struct pasis_cache_s {
    int size;
    int expected_size;
    char fixed_flag;
    unsigned int buckets;
    unsigned int tag_size;
    unsigned int data_size;
    pasis_freelist_t *freelist;
    unsigned int (* hash_fp)(void *tag, unsigned buckets);
    int (* compare_fp)(void *tag1, void *tag2);
    pasis_cache_queue_t *hash_array;
    pasis_cache_queue_t lru_queue;
} pasis_cache_t;

/* Unpins cache object without a remove -- internal lookup done on tag */
void PasisCache_Unpin (pasis_cache_t *cache, void *tag);

/* Pins cache object -- internal lookup done on tag */
void PasisCache_Pin (pasis_cache_t *cache, void *tag);

/* Unpins cache object, no lookup -- data must have been returned by Lookup */
void PasisCache_UnpinData (pasis_cache_t *cache, void *data);
                           
/* Pins cache object, no lookup -- data must have been returned by Lookup */
void PasisCache_PinData (pasis_cache_t *cache, void *data);
                         
/* Gets an empty element from the cache, it associates that element with the 
 * tag.  If the tag exists, the element associated with the tag is returned */
void *PasisCache_GetEmpty (pasis_cache_t *cache, void *tag);
 
/* Removes item from cache if pin count is 0 -- decrements pin count if > 0 */
void PasisCache_Remove (pasis_cache_t *cache, void *tag);

/* Lookup item and optionally pin it in cache (increment ref count) */
void *PasisCache_Lookup (pasis_cache_t *cache, void *tag, int pin);

void PasisCache_Insert (pasis_cache_t *cache, void *tag, void *data);
void PasisCache_Dump (pasis_cache_t *cache, 
			void (*dump_fp)(void *tag, void *data));
void PasisCache_Shutdown (pasis_cache_t *cache);
pasis_cache_t *PasisCache_Create (int expected_size, char fixed_flag,
                                  unsigned int tag_size, 
                                  unsigned int data_size,
                                  unsigned int (* hash_fp)(void *tag, 
                                                           unsigned int buckets),
                                  int (* compare_fp)(void *tag1, void *tag2));
                                                                 

#ifdef __cplusplus
}
#endif

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */  


#endif /* __PASIS_CACHE_H__ */
