/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_DEBUG_H__
#define __PASIS_DEBUG_H__

#include "pasis_config.h"

#ifdef __cplusplus
extern "C" {
#endif


/* Bitwise OR of all debug types that should be printed from PASIS_DPRINTF.
 * This integer is initialied to PASIS_DEBUG_ENABLE (see pasis_config.h).
 * It can be dynamically changed at run-time. */
extern int pasis_debug_print_enable;


/* Boolean flag indicating whether PASIS_PRINT_INFO messages should be printed
 * or not.  Initialized to true. */
extern int pasis_info_print_enable;


/* Set by configure in pasis_config.h */
#if PASIS_DEBUG > 0

# define PASIS_DPRINTF(type, level, strargs...) \
    if ((level <= PASIS_DEBUG_LEVEL) && (type & pasis_debug_print_enable))  {   \
        fprintf(stderr, strargs);                                               \
    }

# define PASIS_DASSERT(exp) assert((exp))

# define PASIS_THROW(exp) \
    {fprintf(stderr, "%s:%u - exception in: %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__); throw exp;}

#else  /* !PASIS_DEBUG */

# define PASIS_DPRINTF(type, level, strargs...)
# define PASIS_DASSERT(exp) assert((exp))
# define PASIS_THROW(exp) throw exp;

#endif /* !PASIS_DEBUG */

#define PASIS_PRINT_INFO(strargs...)                                            \
    if (pasis_info_print_enable) {                                              \
        printf(strargs);                                                        \
    }

#define PASIS_PRINT_INFO_FLUSH()                                                \
    if (pasis_info_print_enable) {                                              \
        fflush(stdout);                                                         \
    }


char*
pasis_ip_to_hostname (unsigned int ip);


#ifdef __cplusplus
}
#endif

#endif /* __PASIS_DEBUG_H__ */


