/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_ERROR_H__
#define __PASIS_ERROR_H__

#include "pasis_config.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned int pasis_error_t;
typedef unsigned int pasis_bool_t;

#define PASIS_TRUE               1
#define PASIS_FALSE              0

#define PASIS_SUCCESS            0
#define PASIS_ERROR              1

/* RPC level errors */
#define PASIS_ERROR_RPC_TERM     350
#define PASIS_ERROR_RPC_ERR      351
#define PASIS_ERROR_RPC_SERV     352

/* Pasis IO errors */
#define PASIS_ERROR_IO_UNAVAIL   400
#define PASIS_ERROR_IO_PERM      401
#define PASIS_ERROR_IO_D_OLD     402
#define PASIS_ERROR_IO_D_DEST    403
#define PASIS_ERROR_IO_NOEXIST   404
#define PASIS_ERROR_IO_RECONS    405
#define PASIS_ERROR_IO_NET       406
#define PASIS_ERROR_IO_INVALID   407
#define PASIS_ERROR_IO_RETRY     408

#define PASIS_ERROR_DRIVE_EXISTS  450
#define PASIS_ERROR_DRIVE_INVALID 451

/* Pasis IO protocol return codes */
#define PASIS_ERROR_PROTO_COMPLETE 0
#define PASIS_ERROR_PROTO_ABORT    500
#define PASIS_ERROR_PROTO_REREAD   501
#define PASIS_ERROR_PROTO_INCOMPLETE  502
#define PASIS_ERROR_PROTO_REPAIR   503
#define PASIS_ERROR_PROTO_NOWRITE  504
#define PASIS_ERROR_PROTO_RETRY    505

/* Pasis FS errors */
#define PASIS_ERROR_NFS          600
#define PASIS_ERROR_NOMEM        601

#define PASIS_ERROR_BUF_INUSE    700

#define PASIS_ERROR_FILE_RELEASE 800
#define PASIS_ERROR_FILE_ISDIR   801
#define PASIS_ERROR_FILE_NOTLNK  802
#define PASIS_ERROR_FILE_NOTREG  803
#define PASIS_ERROR_MOBJ_INVALID 804
#define PASIS_ERROR_LOCK_LOCKED  805

#define PASIS_ERROR_DIR_NOENT    900
#define PASIS_ERROR_DIR_NOSPACE  901
#define PASIS_ERROR_DIR_NOTDIR   902
#define PASIS_ERROR_DIR_NOTEMPTY 903



#define PASIS_ERROR_IO           1000

/* Pasis ED errors */
#define PASIS_ERROR_ED_INTEGRITY   2000
#define PASIS_ERROR_ED_SHARE_INDEX 2001
#define PASIS_ERROR_ED_INVALID_CC  2002

#define PASIS_ERROR_TABLE \
{PASIS_SUCCESS, "Operation was a success."}, \
{PASIS_ERROR, "Generic Error."}, \
{PASIS_ERROR_RPC_TERM, "RPC terminated early."}, \
{PASIS_ERROR_RPC_ERR, "RPC error."}, \
{PASIS_ERROR_RPC_SERV, "RPC server error."}, \
{PASIS_ERROR_IO_UNAVAIL, "Storage node is unavailable."}, \
{PASIS_ERROR_IO_PERM, "Permission denied."}, \
{PASIS_ERROR_IO_D_OLD, "Old data recoverable."}, \
{PASIS_ERROR_IO_D_DEST, "Data destroyed."}, \
{PASIS_ERROR_IO_NOEXIST, "Data does not exist."}, \
{PASIS_ERROR_IO_RECONS, "Data unreconstructable."}, \
{PASIS_ERROR_IO_NET, "Network error."}, \
{PASIS_ERROR_IO_INVALID, "Invalid request."}, \
{PASIS_ERROR_DRIVE_EXISTS, "Drive type already exists."}, \
{PASIS_ERROR_DRIVE_INVALID, "Invalid drive type."}, \
{PASIS_ERROR_PROTO_ABORT, "Protocol read must abort."}, \
{PASIS_ERROR_PROTO_REREAD, "Protocol read must read more."}, \
{PASIS_ERROR_PROTO_INCOMPLETE, "Protocol read found incomplete write."}, \
{PASIS_ERROR_PROTO_REPAIR, "Protocol read found repairable write."}, \
{PASIS_ERROR_PROTO_NOWRITE, "Protocol read found no complete write."}, \
{PASIS_ERROR_NFS, "Generic NFS Error."}, \
{PASIS_ERROR_NOMEM, "Out of memory."}, \
{PASIS_ERROR_BUF_INUSE, "Buffer is currently in use."}, \
{PASIS_ERROR_FILE_RELEASE, "File has been released."}, \
{PASIS_ERROR_FILE_ISDIR, "File is a directory."}, \
{PASIS_ERROR_FILE_NOTLNK, "File is not a symlink."}, \
{PASIS_ERROR_FILE_NOTREG, "File is not a regular file."}, \
{PASIS_ERROR_MOBJ_INVALID, "Invalid metadata object pointer found."}, \
{PASIS_ERROR_LOCK_LOCKED, "Lock is already held."}, \
{PASIS_ERROR_DIR_NOENT, "Directory entry not found."}, \
{PASIS_ERROR_DIR_NOSPACE, "Directory too full."}, \
{PASIS_ERROR_DIR_NOTDIR, "File is not a directory."}, \
{PASIS_ERROR_DIR_NOTEMPTY, "Directory is not empty."}, \
{PASIS_ERROR_IO, "Generic I/O error."}, \
{PASIS_ERROR_ED_INTEGRITY, "Data-item hash failed integrity check."}, \
{PASIS_ERROR_ED_SHARE_INDEX, "Share index is invalid."}, \
{PASIS_ERROR_ED_INVALID_CC, "Cross checksum does not validate."}



/* wrapper around stderror printing... */
#define PASIS_PRINT_ERR(args...) fprintf(stderr, args)
#define PASIS_STRERROR(error) Pasis_StrError(error)

/*********************** Need not edit below this line *********************/

#define PASIS_ERROR_STR_LEN 100

typedef struct error_pair_s {
  pasis_error_t error_code;
  char error_str[PASIS_ERROR_STR_LEN];
} pasis_error_pair_t;

#define PASIS_NUM_ERRORS (sizeof(errorTable) / sizeof(pasis_error_pair_t))
#define PASIS_FIRST_ANSI_ERROR 1
#define PASIS_LAST_ANSI_ERROR 124

static pasis_error_pair_t errorTable[] = {PASIS_ERROR_TABLE};

static inline
const char *Pasis_StrError(pasis_error_t error)     /* the error code */
{
    unsigned int i;

    /* check to see if it is a system error */ 
    if (error >= PASIS_FIRST_ANSI_ERROR &&
	error <= PASIS_LAST_ANSI_ERROR){
        /* return the system error string */ 
        return strerror (error);
    } else {
        /* scan our error code table for the error code. */
        for(i = 0; i < PASIS_NUM_ERRORS; i++){
            if(errorTable[i].error_code == error){
                return errorTable[i].error_str;
            }
        }
        /* if it finished the loop without returning, 
           we know it didn't find the error and the
           error code is unknown. */
        return "Unknown error code";      
    }
}

#ifdef __cplusplus
}
#endif


#endif /* __PASIS_ERROR_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */  
