/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_EVENT_H__
#define __PASIS_EVENT_H__

#include "pasis_config.h"

#ifdef __cplusplus
extern "C" {
#endif

#define PASIS_EVENT_ERROR POLLERR
#define PASIS_EVENT_READ  POLLIN
#define PASIS_EVENT_WRITE POLLOUT

typedef struct pasis_event_callback_s {
    void (*callback)(void *arg);
    void *arg;
} pasis_event_callback_t;

int PasisEvent_Init (void *(*shutdown_fp)(void *));
int PasisEvent_RegisterFd (st_netfd_t st_fd,
                           pasis_event_callback_t *read_cb,
                           pasis_event_callback_t *write_cb,
                           pasis_event_callback_t *error_cb);
int PasisEvent_UnregisterFd (st_netfd_t fd);
int PasisEvent_Unmask (st_netfd_t fd, unsigned int event_mask);
int PasisEvent_Mask (st_netfd_t fd, unsigned int event_mask);
void PasisEvent_Loop (void);
void PasisEvent_Shutdown (void);
void PasisEvent_SignalShutdown (void);
st_thread_t PasisEvent_ThreadStart (void);

#ifdef __cplusplus
}
#endif

#endif /* __PASIS_EVENT_H__ */





