/* Wrap Gladman's code
 */
 
#include "pasis_config.h"
#include "pasis_debug.h"

#include "ed_c_sha.h"
#include "ed_c_sha256_gladman.h"

//See the following for SHA spec:
//http://csrc.nist.gov/encryption/shs/sha256-384-512.pdf

void
ED_C_SHA256_G::digest( const p_byte *buffer,
                       unsigned int length, 
                       p_byte *digest )
{
    sha256_ctx  cx[1];
    sha256_begin(cx); 
    sha256_hash(buffer, length, cx); 
    sha256_end(digest, cx);    
}

void 
ED_C_SHA256_G::digest( const p_byte **buffers,
                       unsigned int *buffer_lengths, 
                       unsigned int num_buffers,
                       p_byte *digest )
{
    sha256_ctx  cx[1];
    sha256_begin(cx); 
    for( unsigned int i = 0; i < num_buffers; ++i ) {
        sha256_hash(buffers[i], buffer_lengths[i], cx); 
    }
    sha256_end(digest, cx);    
}

void 
ED_C_SHA256_G::key_digest( const p_byte *key,
                           unsigned int key_length,
                           const p_byte *buffer,
                           unsigned int buffer_length, 
                           p_byte *digest )
{
    sha256_ctx  cx[1];
    sha256_begin(cx); 
    sha256_hash(key, key_length, cx); 
    sha256_hash(buffer, buffer_length, cx); 
    sha256_end(digest, cx);    
}


void 
ED_C_SHA256_G::key_digest( const p_byte *key,
                           unsigned int key_length,
                           const p_byte **buffers,
                           unsigned int *buffer_lengths, 
                           unsigned int num_buffers,
                           p_byte *digest )
{
    sha256_ctx  cx[1];
    sha256_begin(cx); 
    sha256_hash(key, key_length, cx); 
    for( unsigned int i = 0; i < num_buffers; ++i ) {
        sha256_hash(buffers[i], buffer_lengths[i], cx); 
    }
    sha256_end(digest, cx);    
}

