#include "krb_box.h"
#include "cred_cache.h"
#include "../tools/state_threads/include/st.h"
#include "../common/lib/include/pasis_freelist.h"
#include "../common/lib/include/pasis_lists.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <krb5.h>
#include <com_err.h>

krb5_ccache ccdef;
static krb5_principal my_princ;
static krb5_context context;
static st_thread_t exp_thread;
static st_thread_t req_thread;

static krb5_error_code retval;
static st_cond_t exp_cond, new_req_cond, no_free_req_cond;
/* static u_int cli_addr; */
static char* cli_name = NULL;
static char* progname = "krb keycache";
static int running = 0;
static pasis_freelist_t* req_freelist = NULL;


struct key_request {
    u_int client;
    u_int server;
    octet_string* ret_bytes;
    st_cond_t return_cond;
    PASIS_Q_PTRS(struct key_request) ptrs;
};


PASIS_Q_STRUCT(req_list_t, struct key_request);

struct req_list_t req_list;



/* initializer for key_request objects.
   to be used only by the pasis_freelist functions */
void const_key_request(void* arg) {
    struct key_request* tmp = (struct key_request*) arg;
    tmp->return_cond = st_cond_new();
}

/* destructor for key_request objects.
   to be used only by the pasis_freelist functions */
void dest_key_request(void* arg) {
    struct key_request* tmp = (struct key_request*) arg;
    st_cond_destroy(tmp->return_cond);
}

/* deals with the kerberos to get a shared key
   adds it to the hashtable when done */
int do_get_key(u_int client, u_int server) {

    char full_hname[MAXHOSTNAMELEN];
    char buffer[INPUT_BUF_SIZE];
    st_netfd_t st_outsock;
    struct sockaddr_in s_sock;		/* server address */
    struct sockaddr_in c_sock;		/* client address */
    struct hostent *host = NULL;
    int i;
    int outsock;
    krb5_data packet, inbuf;
    krb5_data ret_packet;
    krb5_ap_rep_enc_part *reply = NULL;
    krb5_keyblock *session_keyblock = NULL;
    krb5_creds incred, outcred;
    char* service = SERVICE_NAME;
    krb5_auth_context auth_context = NULL;
    char* cp = NULL;
    
    printf("do get key %d %d\n", client, server);
            

    /* Look up server host */
    if ((host = gethostbyaddr(&server, sizeof(server), AF_INET)) == (struct hostent *) 0) {
	fprintf(stderr, "%d: unknown host\n", server);
	exit(1);
    }
    strncpy(full_hname, host->h_name, sizeof(full_hname)-1);
    full_hname[sizeof(full_hname)-1] = '\0';

    /* lower-case to get name for "instance" part of service name */
    for (cp = full_hname; *cp; cp++)
        if (isupper(*cp))
            *cp = tolower(*cp);

    /* Set server's address */
    (void) memset((char *)&s_sock, 0, sizeof(s_sock));

    memcpy((char *)&s_sock.sin_addr, host->h_addr, sizeof(s_sock.sin_addr));

    s_sock.sin_family = AF_INET;
    s_sock.sin_port = htons(AUTH_PORT);

    inbuf.data = full_hname;
    inbuf.length = strlen(full_hname);
    printf("host %s\n", full_hname);
    if ((retval = krb5_mk_req(context, &auth_context, AP_OPTS_MUTUAL_REQUIRED,
                              service, full_hname,
			      &inbuf, ccdef, &packet))) {
        krb5_err(context, 1, retval, "while preparing AP_REQ");
        //exit(1);
    }
    printf("Got credentials for %s.\n", service);

 
    /* Open a socket */

    if ((outsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        com_err(progname, errno, "opening datagram socket");
        exit(1);
    } 
    st_outsock = st_netfd_open(outsock);

    memset((char *)&c_sock, 0, sizeof(c_sock));
    c_sock.sin_family = AF_INET;

    /* Bind it to set the address; kernel will fill in port # */
    if (bind(outsock, (struct sockaddr *)&c_sock, sizeof(c_sock)) < 0) {
	com_err(progname, errno, "while binding datagram socket");
	exit(1);
    }
    
    /* done with sockets crap */
    krb5_auth_con_getkey(context, auth_context, &session_keyblock);
    printf("client key = ");
    for (i = 0; i<session_keyblock->keyvalue.length; i++)
	printf("%02x", ((unsigned char*)session_keyblock->keyvalue.data)[i]);
    printf("\n");



    /* Send authentication info to server */
    if ((i = st_sendto(st_outsock, (void *)packet.data, packet.length,
		       (struct sockaddr*) &s_sock, sizeof(struct sockaddr_in), NET_TO)) < 0) {
        if (errno == EINTR)
            return -1;
        else
            com_err(progname, errno, "while sending KRB_AP_REQ message");
    }
    printf("Sent authentication data: %d bytes\n", i);

    if ((i = st_recvfrom(st_outsock, buffer, 
                         INPUT_BUF_SIZE, NULL, 0, NET_TO)) < 0) {
        if (errno == EINTR)
            return -1;
        else {
            com_err(progname, errno, "while recieving reply");
            exit(1);
        }
    }

    ret_packet.data = &buffer;
    ret_packet.length = i;

    if ((retval = krb5_rd_rep(context, auth_context, &ret_packet, &reply)) < 0) {
        com_err(progname, errno, "while unpacking reply");
        exit(1);
    }

    krb5_auth_con_getkey(context, auth_context, &session_keyblock);
    
    krb5_sname_to_principal(context, full_hname, service, 0, &incred.server);

    krb5_cc_retrieve_cred(context, ccdef, 0, &incred, &outcred);

    //printf("inserting key\n");
    hash_item(cli_name, client, s_sock.sin_addr.s_addr, outcred.times.endtime, session_keyblock->keyvalue.length,
              session_keyblock->keyvalue.data);

    printf("inserted key for %s %d %d\n", cli_name, client,s_sock.sin_addr.s_addr );
    
    
     /* kick the expiration thread */
    st_cond_signal(exp_cond);

    /* free everything */
    krb5_free_data_contents(context, &packet);
    //krb5_free_data_contents(context, &inbuf);
    krb5_free_ap_rep_enc_part(context, reply);
    krb5_free_keyblock(context, session_keyblock);
    krb5_free_creds_contents(context, &incred);
    krb5_free_creds_contents(context, &outcred);
    krb5_auth_con_free(context, auth_context);    
    
    /* close socket */
    st_netfd_close(st_outsock);

    return 0;
}

/* handle incoming requests in a seperate thread */
void* req_handler(void* arg) {
    struct key_request* req = NULL;

    printf("starting request handler\n");

    while (running) {
        if (PASIS_Q_EMPTY(req_list)) {
            if (st_cond_wait(new_req_cond) < 0) {
                if (errno == EINTR) {
                    return NULL;
                } else
                    perror("error in cond_wait");
            }
        }
        
        else {
            req = PASIS_Q_FIRST(req_list);
            do_get_key(req->client, req->server);
            st_cond_signal(req->return_cond);

            PASIS_Q_REMOVE(req_list, ptrs, req);

            PasisFreelist_Put(req_freelist, (void*)req);
            st_cond_signal(no_free_req_cond);
        }
    }

    return NULL;
}

/* handle expiring keys in a seperate thread */        
void* exp_handler(void* arg) {

    time_t now = time(NULL);
    struct hash_entry *nextexp = NULL;
    time_t nexttime = 0;
   
    printf("expire handler started\n");

    while (running) {
        nexttime = hash_nextexp_time();
        //printf("next expire in %d %d\n", (int) (nexttime-now), (int) nexttime);
        while ((nexttime-now) > 60 || nexttime == -1) {
            if (nexttime == -1) {
                if (st_cond_wait(exp_cond) < 0) {
                    if (errno == EINTR) {
                        /* st_cond_broadcast(return_cond); */
                        return NULL;
                    } else
                        perror("error in cond_wait");
                }
            } else {
                if (st_cond_timedwait(exp_cond, (nexttime-now)*500000) < 0) {
                    if (errno == EINTR) {
                        /* st_cond_broadcast(return_cond); */
                        return NULL;
                    } else
                        perror("error in cond_wait");
                }
            }    
            now = time(NULL);
            nexttime = hash_nextexp_time();
        }
        
        //printf("hashget\n");
        nextexp = hash_nextexp();

        if (nextexp) {
            do_get_key(nextexp->client, nextexp->server);
            free(nextexp->princ);
            free(nextexp->key);
            free(nextexp);
            nextexp = NULL;
            /* st_cond_broadcast(return_cond); */
        } else {
            printf("WERT! NULL!\n");
        }
    }

    return NULL;
    
}

/* do kerberos and thread init */
int krb_init_keycache_cli() {
    char my_hostname[MAXHOSTNAMELEN+1];
    struct hostent *host;

    if (running) {
        printf("already initialized!\n");
        return -1;
    }


    retval = krb5_init_context(&context);
    printf("initializing kerberos\n");
    if (retval) {
        com_err(progname, retval, "while initializing krb5");
        exit(1);
    }
        
    hash_init();
    //st_init();
    exp_cond = st_cond_new();
    new_req_cond = st_cond_new();
    no_free_req_cond = st_cond_new();
    
    req_freelist = PasisFreelist_CreateInitFixed(sizeof(struct key_request), MAX_REQUESTS,
                                  const_key_request, dest_key_request);

    PASIS_Q_INIT(req_list);

    /* Get credentials cache */
    if ((retval = krb5_cc_default(context, &ccdef))) {
        krb5_err(context,1, retval, "while getting default ccache");
        exit(1);
    }
    
    /* get my principal */
    if ((retval = krb5_cc_get_principal(context, ccdef, &my_princ))) {
        krb5_err(context,1,  retval, "while getting my principal");
        exit(1);
    }
    
    if ((retval = krb5_unparse_name(context, my_princ,
				    &cli_name))) {
	krb5_err(context,1,  retval, "while unparsing client name");
	exit(1);
    }
    
    gethostname(my_hostname, MAXHOSTNAMELEN);
    if ((host = gethostbyname(my_hostname)) == (struct hostent *)0) {
	fprintf(stderr, "%s: host unknown\n", my_hostname);
	exit(1);
    }
    
    //cli_addr = inet_addr(host->h_addr);
    running = 1;

    exp_thread = st_thread_create(exp_handler, NULL, 1, 1<<18);
    req_thread = st_thread_create(req_handler, NULL, 1, 1<<18);
    return 0;
    
}


/* main API function handles request into the library */
octet_string* krb_get_sh_key_cli(char* princ, u_int client, u_int server) {

    struct hash_entry* hash_found = NULL;
    octet_string* ret_key = NULL;
    struct key_request* req = NULL;

    if (!running) {
        printf("not initialized!\n");
        return NULL;
    }
    printf("trying to get key for %s %d %d\n", princ, client, server);
    


    hash_found = hash_get(princ, client, server);

        
    if (hash_found && hash_found->keysize != 0) {
        printf("found\n");
        ret_key = (octet_string*) malloc(sizeof(octet_string));
        ret_key->length = hash_found->keysize;
        ret_key->data = (byte*) malloc(ret_key->length*sizeof(byte));
        memcpy(ret_key->data, hash_found->key, ret_key->length);
    } else {
        //do_get_key(client, server);
        /*printf("poking freelist\n");*/
        while ((req = PasisFreelist_Get(req_freelist)) == NULL) {
            printf("freelist full... waiting\n");
            st_cond_wait(no_free_req_cond);
        }
         
        /*printf("got freelist pointer 0x%x\n", (unsigned int)req);*/
        
        req->client = client;
        req->server = server;
        PASIS_Q_ADD_TAIL(req_list, ptrs, req);
        st_cond_signal(new_req_cond);

        st_cond_wait(req->return_cond);
        hash_found = hash_get(princ, client, server);
        
        if (hash_found && hash_found->keysize != 0) {
            printf("found\n");
            ret_key = (octet_string*) malloc(sizeof(octet_string));
            ret_key->length = hash_found->keysize;
            ret_key->data = (byte*) malloc(ret_key->length*sizeof(byte));
            memcpy(ret_key->data, hash_found->key, ret_key->length);
        } else {
            ret_key = NULL;
        }
            
    }
    
    return ret_key;

}

/* shutodwn, kill threads, and free memory */
int krb_shutdown_cli(void) {

    if (!running) {
        printf("not initialized!\n");
        return -1;
    }
    running = 0;
    st_thread_interrupt(exp_thread);
    st_thread_join(exp_thread, NULL);
    st_thread_interrupt(req_thread);
    st_thread_join(req_thread, NULL);

    st_cond_destroy(exp_cond);
    st_cond_destroy(new_req_cond);
    st_cond_destroy(no_free_req_cond);

    PasisFreelist_Destroy(req_freelist);

    hash_shutdown();
    krb5_free_principal(context, my_princ);
    krb5_cc_close(context, ccdef);
    krb5_free_context(context);
    
    return 0;
}



/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
