/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASISFS_NFS3_FE_H__
#define __PASISFS_NFS3_FE_H__

#include "pasis_config.h"
#include "pasis_types.h"

#include "pasis_md_prot.h"

#include "mount3_prot.h"
#include "nfs3_prot.h"

int PasisFS_NFS3FeInit (pasis_md_protocol_t *md_proto, 
                        unsigned int         md_num_linkages,
                        pasis_md_linkage_t  *md_linkages,
                        pasis_md_protocol_t *data_proto,
                        unsigned int         data_num_linkages,
                        pasis_md_linkage_t  *data_linkages);

void PasisFS_NFS3FeShutdown (void);

nfsstat3 PasisFS_NFS3GetRoot (unsigned int uid, nfspath3 path, nfs_fh3 *fhout);

nfsstat3 PasisFS_NFS3GetRoot (unsigned int uid, nfspath3 path, nfs_fh3 *fhout);

nfsstat3 PasisFS_NFS3GetAttr (unsigned int uid, nfs_fh3 *fh, fattr3 *fa_out);

nfsstat3 PasisFS_NFS3SetAttr (unsigned int uid, nfs_fh3 *fh, sattr3 *sa_in,
			      sattrguard3 *guard, wcc_data *wcc);

nfsstat3 PasisFS_NFS3Lookup (unsigned int uid, nfs_fh3 *dir, filename3 file, 
			     nfs_fh3 *file_out, post_op_attr *obj_attr_out,
			     post_op_attr *dir_attr_out, 
			     post_op_attr *fail_attr);

nfsstat3 PasisFS_NFS3Access (unsigned int uid, nfs_fh3 *fh, uint32_t access_in,
			     post_op_attr *obj_attr_out, uint32_t *access_out);

nfsstat3 PasisFS_NFS3Readlink (unsigned int uid, nfs_fh3 *fh, 
			       nfspath3 path_out, post_op_attr *sym_attr_out);

nfsstat3 PasisFS_NFS3Read (unsigned int uid, nfs_fh3 *fh, uint64_t offset,
			   uint32_t len, char *buffer, uint32_t *len_out, 
			   bool_t *eof, post_op_attr *file_attr_out);

nfsstat3 PasisFS_NFS3Write (unsigned int uid, nfs_fh3 *fh, uint64_t offset, 
			    uint32_t len, char *buffer, stable_how stable,  
			    uint32_t *len_out, wcc_data *wcc,
			    stable_how *committed, writeverf3 verf_cookie);

nfsstat3 PasisFS_NFS3Create (unsigned int uid, unsigned int gid, nfs_fh3 *dir,
			     filename3 name, createmode3 mode, sattr3 *sa_in, 
			     diropres3ok *dirres);

nfsstat3 PasisFS_NFS3Mkdir (unsigned int uid, unsigned int gid, nfs_fh3 *dir, 
			    filename3 name, sattr3 *sa_in, 
			    diropres3ok *dirres);

nfsstat3 PasisFS_NFS3Symlink (unsigned int uid, unsigned int gid, 
			      nfs_fh3 *from_dir, filename3 from_name, 
			      nfspath3 link, sattr3 *sa_in, 
			      diropres3ok *dirres);

nfsstat3 PasisFS_NFS3Mknod (unsigned int uid, unsigned int gid, nfs_fh3 *dir,
			    filename3 name, mknoddata3 *mknod_data, 
			    diropres3ok *dirres);

nfsstat3 PasisFS_NFS3Unlink (unsigned int uid, nfs_fh3 *dir, filename3 name,
			     wcc_data *wcc);

nfsstat3 PasisFS_NFS3Rmdir (unsigned int uid, nfs_fh3 *dir, filename3 name,
			    wcc_data *wcc);

nfsstat3 PasisFS_NFS3Rename (unsigned int uid, nfs_fh3 *from_dir, 
			     filename3 from_name, nfs_fh3 *to_dir, 
			     filename3 to_name, wcc_data *fromdir_wcc, 
			     wcc_data *todir_wcc);

nfsstat3 PasisFS_NFS3Link (unsigned int uid, nfs_fh3 *from_fh, nfs_fh3 *to_dir,
			   filename3 to_name, post_op_attr *file_attr_out,
			   wcc_data *linkdir_wcc);

nfsstat3 PasisFS_NFS3Readdir (unsigned int uid, nfs_fh3 *dir, uint32_t count, 
			      uint64_t cookie, cookieverf3 cookieverf_in, 
			      post_op_attr *post_dir_attr, 
			      cookieverf3 cookieverf_out, struct iovec *iov);

nfsstat3 PasisFS_NFS3Readdirplus (unsigned int uid, nfs_fh3 *dir, 
				  uint32_t count, uint32_t dircount, 
				  uint64_t cookie, cookieverf3 cookieverf_in, 
				  post_op_attr *post_dir_attr, 
				  cookieverf3 cookieverf_out, 
                                  struct iovec *iov);

nfsstat3 PasisFS_NFS3FSstat (unsigned int uid, nfs_fh3 *fh, 
			     fsstat3resok *stat);

nfsstat3 PasisFS_NFS3FSinfo (unsigned int uid, nfs_fh3 *fh,
			     fsinfo3resok *info);

nfsstat3 PasisFS_NFS3Pathconf (unsigned int uid, nfs_fh3 *fh, 
			       pathconf3resok *pathconf);

nfsstat3 PasisFS_NFS3Commit (unsigned int uid, nfs_fh3 *fh, uint64_t offset, 
			     uint32_t count, wcc_data *file_wcc, 
			     writeverf3 writeverf_out);

#endif /* __PASISFS_NFS3_FE_H__ */
