/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASISFS_DATA_CACHE_H__
#define __PASISFS_DATA_CACHE_H__

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasis_lists.h"

#include "fe.h"
#include "pasis_md_prot.h"
#include "pasisio_md_protocol.h"

#include "pasisfs_buf_cache.h"

/* Size of data cache determined by buffer pool size passed into appliation */

#define PASISFS_DCACHE_DATA(bp) ((pasisfs_dcache_data_t *)bp->b_data)
#define PASISFS_DCACHE_TAG(bp) ((pasisfs_dcache_tag_t *)bp->b_tag)

#define PASISFS_DCACHE_BLKNO(bp) (((pasisfs_dcache_tag_t *)bp->b_tag)->blkno)

typedef p_uint64 pasisfs_blkaddr_t;

typedef struct pasisfs_dcache_tag_s {
    pasis_md_fid_t     fid;
    pasisfs_blkaddr_t  blkno;
} pasisfs_dcache_tag_t;

typedef struct pasisfs_dcache_data_s {
    pasis_md_loc_t    loc;
    time_t            evict_time;
    p_byte           *data;

    PASIS_Q_PTRS (struct pasisfs_buf_s) b_list;
} pasisfs_dcache_data_t;

/* Get empty buffer, bp returned is pinned */
pasisfs_buf_t *PasisFS_DataCacheGetEmpty (void);

/* Insert block into cache, bp must be pinned */
pasis_error_t PasisFS_DataCacheInsert (pasis_md_fid_t *fid,
                                       pasis_md_loc_t *loc,
                                       p_uint64 blkno, 
                                       pasisfs_buf_t *bp);

/* Write out buffer, bp must be pinned */
pasis_error_t PasisFS_DataCacheWrite (pasisfs_buf_t *bp);

/* Read in data buffer, bp returned is pinned */
pasisfs_buf_t *PasisFS_DataCacheRead (pasis_md_fid_t *fid,
                                      pasis_md_loc_t *loc,
                                      pasisfs_blkaddr_t blkno);

/* Does a find in core, bp returned is pinned if not NULL */
pasisfs_buf_t *PasisFS_DataCacheLookup (pasis_md_fid_t *fid,
                                        pasisfs_blkaddr_t blkno);

/* Pass in pre-allocated memory and the size of that memory */
pasis_error_t PasisFS_DataCacheInit (p_byte *data_pool, unsigned int size);

void PasisFS_DataCacheShutdown (void);


#endif /* __PASISFS_DATA_CACHE_H__ */


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */  
