/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __NFS_FE_RPC_H__
#define __NFS_FE_RPC_H__

#include "pasis_config.h"

#define RPC_MAX_CONNECTIONS (256)
#define RPC_MAX_PAYLOAD     (32 << 10) /* 32KB */
#define RPC_MAX_HDR         (1024)

#define RPC_MAX_PACKET      (RPC_MAX_PAYLOAD + RPC_MAX_HDR)

#define RPC_ACCEPT_TIMEOUT      5000000LL  /* TCP accept timeout in usecs */
#define RPC_REQ_TIMEOUT         5000000LL  /* TCP rpc request timeout usecs */

#define RPC_AUTH_NONE       0
#define RPC_AUTH_NULL       0
#define RPC_AUTH_UNIX       1

#define RPC_MESG_VERSION     2

typedef enum rpc_msg_type {
        RPC_CALL=0,
        RPC_REPLY=1
} rpc_msg_type_t;

typedef enum rpc_reply_stat {
        RPC_MSG_ACCEPTED=0,
        RPC_MSG_DENIED=1
} rpc_reply_stat_t;

typedef enum rpc_accept_stat {
        RPC_ACCEPT_SUCCESS=0,
        RPC_PROG_UNAVAIL=1,
        RPC_PROG_MISMATCH=2,
        RPC_PROC_UNAVAIL=3,
        RPC_GARBAGE_ARGS=4,
        RPC_SYSTEM_ERR=5
} rpc_accept_stat_t;

typedef enum rpc_reject_stat {
        RPC_REJECT_MISMATCH=0,
        RPC_AUTH_ERROR=1
} rpc_reject_stat_t;

typedef enum rpc_auth_stat {
        RPC_AUTH_OK=0,
        /* failed at remote end */
        RPC_AUTH_BADCRED=1,             /* bogus credentials (seal broken) */
        RPC_AUTH_REJECTEDCRED=2,        /* client should begin new session */
        RPC_AUTH_BADVERF=3,             /* bogus verifier (seal broken) */
        RPC_AUTH_REJECTEDVERF=4,        /* verifier expired or was replayed */
        RPC_AUTH_TOOWEAK=5,             /* rejected due to security reasons */
        /* failed locally */
        RPC_AUTH_INVALIDRESP=6,         /* bogus response verifier */
        RPC_AUTH_FAILED=7               /* some unknown reason */
} rpc_auth_stat_t;


typedef struct rpc_state_s {
    u_int32_t         rpc_xid;
    u_int32_t         rpc_prog;
    u_int32_t         rpc_vers;
    u_int32_t         rpc_proc;
    u_int32_t         rpc_auth_uid;
    u_int32_t         rpc_auth_gid;

    unsigned long     rpc_proto;

    struct sockaddr_in rpc_addr;
    st_netfd_t         rpc_fd;   

    /* Reply status codes */ 
    rpc_reply_stat_t  reply_status; /* 0 - ACCEPTED, 1 - DENIED */
    /* For denied reply statuses */
    rpc_reject_stat_t reply_rejected_status; /* 0 - RPC, 1 - AUTH */
    rpc_auth_stat_t   reply_rejected_auth_status; /* If rejected stat == 1 */
    /* For accepted reply statuses */
    rpc_accept_stat_t reply_accepted_status; /* 0 - SUCCESS, 1-5 - FAILURES */ 


    /* RPC buffers allocated at started, fixed addresses */
    char         *in_buffer;
    char         *out_buffer;

    /* Input/Output iovec, index 0 holds rpc headers, index 1 holds data */
    struct iovec  in_iov[2];
    struct iovec  out_iov[3];
} rpc_state_t;

pasis_error_t rpc_read_udp_request (rpc_state_t *rpc_state);
pasis_error_t rpc_send_udp_reply (rpc_state_t *rpc_state, st_mutex_t lock);

pasis_error_t rpc_read_tcp_request (rpc_state_t *rpc_state);
pasis_error_t rpc_send_tcp_reply (rpc_state_t *rpc_state, st_mutex_t lock);

pasis_error_t rpc_decode_call (rpc_state_t *rpc_state);
void rpc_encode_reply(rpc_state_t *rpc_state);


#endif /* __NFS_FE_RPC_H__ */


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
