/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __XDR_NFS3_H__
#define __XDR_NFS3_H__

#include "pasis_config.h" /* For types: u_int32_t, u_int64_t */
#include "nfs3_prot.h"

/* Decoding arguments */
u_int32_t *xdr_decode_dirop (u_int32_t *p, diropargs3 *dirop);
u_int32_t *xdr_decode_fh (u_int32_t *p, nfs_fh3 *fh);
u_int32_t *xdr_decode_setattr (u_int32_t *p, setattr3args *args);
u_int32_t *xdr_decode_access (u_int32_t *p, access3args *args);
u_int32_t *xdr_decode_read (u_int32_t *p, read3args *args);
u_int32_t *xdr_decode_write (u_int32_t *p, write3args *args);
u_int32_t *xdr_decode_create (u_int32_t *p, create3args *args);
u_int32_t *xdr_decode_mkdir (u_int32_t *p, mkdir3args *args);
u_int32_t *xdr_decode_symlink (u_int32_t *p, symlink3args *args);
u_int32_t *xdr_decode_mknod (u_int32_t *p, mknod3args *args);
u_int32_t *xdr_decode_rename (u_int32_t *p, rename3args *args);
u_int32_t *xdr_decode_link (u_int32_t *p, link3args *args);
u_int32_t *xdr_decode_readdir (u_int32_t *p, readdir3args *args);
u_int32_t *xdr_decode_readdirplus (u_int32_t *p, readdirplus3args *args);
u_int32_t *xdr_decode_commit (u_int32_t *p, commit3args *args);

/* Encoding results */
u_int32_t *xdr_encode_wccstat (u_int32_t *p, wccstat3 *res);
u_int32_t *xdr_encode_dirop (u_int32_t *p, diropres3 *res);
u_int32_t *xdr_encode_getattr (u_int32_t *p, getattr3res *res);
u_int32_t *xdr_encode_lookup (u_int32_t *p, lookup3res *res);
u_int32_t *xdr_encode_access (u_int32_t *p, access3res *res);
u_int32_t *xdr_encode_readlink (u_int32_t *p, readlink3res *res);
u_int32_t *xdr_encode_read (u_int32_t *p, read3res *res);
u_int32_t *xdr_encode_write (u_int32_t *p, write3res *res);
u_int32_t *xdr_encode_rename (u_int32_t *p, rename3res *res);
u_int32_t *xdr_encode_link (u_int32_t *p, link3res *res);
u_int32_t *xdr_encode_readdir_end (u_int32_t *p, int eof);
u_int32_t *xdr_encode_entry (u_int32_t *p, entry3 *entry);
u_int32_t *xdr_encode_readdirplus_end (u_int32_t *p, int eof);
u_int32_t *xdr_encode_entryplus (u_int32_t *p, entryplus3 *entry);
u_int32_t *xdr_encode_readdir (u_int32_t *p, readdir3res *res);
u_int32_t *xdr_encode_readdirplus (u_int32_t *p, readdirplus3res *res);
u_int32_t *xdr_encode_fsstat (u_int32_t *p, fsstat3res *res);
u_int32_t *xdr_encode_fsinfo (u_int32_t *p, fsinfo3res *res);
u_int32_t *xdr_encode_pathconf (u_int32_t *p, pathconf3res *res);
u_int32_t *xdr_encode_commit (u_int32_t *p, commit3res *res);


#endif /* __XDR_NFS3_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */

