/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#include "pasis_config.h"

#include "mount1_prot.h"
#include "fe_xdr.h"

#include "xdr_mount1.h"

/* Decode ops */
u_int32_t *
xdr_decode_dirpath (u_int32_t *p, dirpath *path)
{
    p = xdr_decode_string (p, path);
    return p;
}

static inline u_int32_t *
xdr_encode_fhandle (u_int32_t *p, fhandle *fh)
{
    memcpy ((char *)p, *fh, FHSIZE);
    return p + XDR_QUADLEN (FHSIZE);
}

u_int32_t *
xdr_encode_fhstatus (u_int32_t *p, fhstatus *res)
{
    *p++ = htonl (res->fhs_status);
    if (res->fhs_status == 0) {
        p = xdr_encode_fhandle (p, &res->fhstatus_u.fhs_fhandle);
    }
    return p;
}

static inline u_int32_t *
xdr_encode_groupnode (u_int32_t *p, groupnode *node)
{
    while (node) {
        *p++ = htonl (1);
        p = xdr_encode_string (p, node->gr_name);
        node = node->gr_next;
    }
    *p++ = 0;
    return p;
}

u_int32_t *
xdr_encode_exportnode (u_int32_t *p, exportnode *node)
{
    while (node) {
        *p++ = htonl (1);
        p = xdr_encode_string (p, node->ex_dir);
        p = xdr_encode_groupnode (p, node->ex_groups);
        node = node->ex_next;
    }
    *p++ = 0;
    return p;
}

u_int32_t *
xdr_encode_mountbody (u_int32_t *p, mountbody *body)
{
    while (body) {
        *p++ = htonl (1);
        p = xdr_encode_string (p, body->ml_hostname);
        p = xdr_encode_string (p, body->ml_directory);
        body = body->ml_next;
    }
    *p++ = 0;
    return p;
}

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
