/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASISIO_H__
#define __PASISIO_H__

#include "pasis_types.h"
#include "pasis_error.h"

#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif


/** The maximumn number of servers.
 ** Need to keep ED_SCHEME_MAX_N >= this value.  See ed_schemes.h.
 ** */
#define PASIS_MAX_NUM_SERVERS 30


/** We support two hash functions (although it's trivial to add more): MD5 and
 ** SHA1.  The Q/U paper used MD5, so we make that the default. */
#if 1
#define HASH_FUNCTION ED_C_MD5
#define PASIS_HASH_SIZE 16
#else
#define HASH_FUNCTION ED_C_SHA1
#define PASIS_HASH_SIZE 20
#endif



/** The size of a PASIS fileID (in bytes) -- must be a multiple of 4 */
#define PASISIO_FILEID_SIZE    16



/** 
 * Pasis IO file id structure 
 */
typedef struct pasisio_fid_s {
    p_byte data[PASISIO_FILEID_SIZE];
} pasisio_fid_t;



/**
 * Type of quorum
 */
typedef enum pasisio_quorum_e {
    PASISIO_QUORUM_NONE = 0,              // no quorums; write-all, read-some
    PASISIO_QUORUM_THRESHOLD_COMP_QC_B_T, // thresh. quorum, #-issue,complete=Qc+b+t
    PASISIO_QUORUM_THRESHOLD_COMP_QC_B,   // thresh. quorum, #-issue,complete=Qc+b
    PASISIO_QUORUM_TREE,                  // tree quorum
} pasisio_quorum_t;


/**
 * Storage-node location information -- linkage record 
 */
typedef struct pasisio_linkage_s {
    p_uint32 ip_address;
    p_uint32 partition;
} pasisio_linkage_t;

typedef p_uint32   p_uid_t;               /**< Pasis IO user id */



/**
 * Library startup function.
 * This must be called before all other functions.  It prepares PASISIO
 * to serve requests.
 * @param max_outstanding Maximum number of outstanding PASISIO requests
 */
pasis_error_t PasisIO_Init(unsigned int max_outstanding,
                           unsigned int block_size);


/**
 * Library shutdown function.
 * This shuts down PASISIO.  When it returns, all shutdown will be complete.
 */
void PasisIO_Shutdown(void);



#ifdef __cplusplus
}
#endif

#endif /* __PASISIO_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */ 
