/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

/*
 * pasis_config_parse.h
 *
 * author: Michael Abd-El-Malek
 *
 * Functions for parsing a PASIS configuration file.  See
 * apps/pasisio_sanity_test.cfg for a sample configuration file.
 */

#ifndef __PASIS_CONFIG_PARSE_H__
#define __PASIS_CONFIG_PARSE_H__

#include "pasis_config.h"
#include "pasis_debug.h"
#include "pasis_types.h"
#include "pasisio_drives.h"
#include "pasis_md_prot.h"

#include <stdio.h>
#include <unistd.h>

#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif


typedef struct {
    p_uint32  ip_address;
    char      base_path[16]; /* usually "/X", where X is partition # */
} pasis_config_server_t;


typedef struct {
    p_uint32                num_servers;
    pasis_config_server_t  *servers;
} pasis_config_server_list_t;


/* Reads a configuration file "filename", and sets the "protocol" and
 * "server_list" arguments based on the configuration file.
 * The "servers" list in the "servers_list" argument is allocated by this
 * function using malloc - it's up to the caller to free that memory.
 * If "file_offset" is not null, it will be set to the file offset within the
 * configuration file where this function has stopped reading.  This allows
 * callers to continue parsing the rest of the file.
 * If "line_num" is not null, it will be set to the line number within the
 * configuration file where this function has stopped reading.
 */
pasis_error_t
PasisIO_ParseConfigurationFile (char                       *filename,
                                pasis_md_protocol_t        *protocol,
                                pasis_config_server_list_t *server_list,
                                long                       *file_offset,
                                int                        *line_num);



/* Adds the list of servers in "server_list" to the PASIS drives module.
 * drive_type_index can be found in pasisio_drives.h as PASISIO_DRIVE_TYPE_...
 * If "linkages" is not null, it is assumed that it points to a list of
 * pasisio_linkage_t structures, which will be filled in by this function.
 * It is assumed that the caller is passing in a linkage list of size
 * PASIS_MAX_NUM_SERVERS.
 * If "instances" is not null, it is assumed that it points to a list of
 * pasisio_drive_instance_t pointers, which will be filled in by this function.
 * It is assumed that the caller is passing in a drive instance pointer list of
 * size ED_SCHEME_MAX_N.
*/ 
pasis_error_t
PasisIO_ConnectToStorageNodes (pasis_config_server_list_t *server_list,
                               unsigned int                drive_type_index,
                               pasisio_linkage_t          *linkages,
                               pasisio_drive_instance_t  **instances);


/* Converts a server list to a list of linkage records.  Unlike the top
 * function, this function does NOT connect to the storage nodes.  */
void
PasisIO_ServerListToLinkageRecords (pasis_config_server_list_t *server_list,
                                    pasisio_linkage_t          *linkages);


#ifdef __cplusplus
}
#endif


#endif /* __PASIS_CONFIG_PARSE_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
