/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASISIO_DRIVE_MD_H__
#define __PASISIO_DRIVE_MD_H__

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasis_error.h"
#include "pasis_timer.h"

#include "pasisio_drives.h"

#include "rpc_common.h"

#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif

#define MD_WORKER_THREADS 16
#define MD_DEFAULT_GID    100

#define MD_RPC_ERROR      1
#define MD_SERVER_ERROR   2
#define MD_RPC_TERM       3

/** MD private protocol drive op codes */
#define PASISIO_MD_DRIVE_P_WRITE_OP      PASISIO_DRIVE_OP_RESV1
#define PASISIO_MD_DRIVE_P_READ_OP       PASISIO_DRIVE_OP_RESV2


typedef p_uint32 pasis_fe_part_t;

/** MD drive instance stats */
typedef struct pasisio_drive_md_stats_s {
    pasis_timer_state_t *timers;
    unsigned int         max_in_progress;
    unsigned int         in_progress;
    unsigned int         read_hist;
} pasisio_drive_md_stats_t;


/** MD private structure */
typedef struct pasisio_drive_md_s {
    pasisio_drive_type_t    *drive_type;

    rpc_handle_t            *md_client;

    st_thread_t              reconnect_thread;
    st_cond_t                shutdown_cond;
    int                      shutdown_flag;
    
    p_uint32                 ip_address;       /**< Server ip address */
    pasis_md_part_t          partition;        /**< Server partition */
    
    pasisio_drive_md_stats_t stats;
} pasisio_drive_md_t;

/** Init MD drive */
pasis_error_t
PasisIO_DriveInitMD (void);


/** Shutdown MD drive */
void
PasisIO_DriveShutdownMD (void);


#ifdef __cplusplus
}
#endif

#endif /* __PASISIO_DRIVE_MD_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */ 
