/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASISIO_MD_HIST_CACHE_H__
#define __PASISIO_MD_HIST_CACHE_H__

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasis_error.h"

#include "pasisio.h"

#include "pasis_md_prot.h"
#include "pasis_md_hist_matrix.h"


#define HIST_MATRIX_CACHE_SIZE    (1024)

typedef pasis_md_fid_t pasisio_md_hist_tag_t;

typedef struct md_hist_matrix_data_s {
    pasis_md_fid_t          fid;    /* For debugging */
    unsigned int            ts_num;
    pasis_md_hts_t          ts_vector[PASIS_MD_HIST_MAX_VERSIONS];
    p_uint64                ts_bitmap[PASIS_MD_HIST_MAX_VERSIONS];
    pasis_md_auth_t         authenticators[PASIS_MAX_NUM_SERVERS][PASIS_MAX_NUM_SERVERS];
    unsigned int            auth_num[PASIS_MAX_NUM_SERVERS];
} pasisio_md_hist_data_t;


pasis_error_t
PasisIO_MDHistCacheInit (unsigned int num_outstanding);

void
PasisIO_MDHistCacheShutdown (void);

pasisio_md_hist_data_t *
PasisIO_MDHistCacheLookup (pasisio_md_hist_tag_t  *tag);

pasisio_md_hist_data_t *
PasisIO_MDHistCacheGetEmpty (pasisio_md_hist_tag_t *tag);

void
PasisIO_MDHistCachePin (pasisio_md_hist_data_t *data);

void
PasisIO_MDHistCacheUnpin (pasisio_md_hist_data_t *data);

void
PasisIO_MDHistCacheUpdate (pasis_md_hist_vector_t *vector,
                           pasisio_md_hist_data_t *data);

void
PasisIO_MDHistCacheSetMatrix (pasisio_md_hist_data_t *data,
                              pasis_md_hist_matrix_t *matrix);

void
PasisIO_MDHistCacheSetAuths (pasis_md_auth_t  auth_to_send[PASIS_MAX_NUM_SERVERS]
                                                          [PASIS_MAX_NUM_SERVERS],
                             unsigned int     auth_to_send_num[PASIS_MAX_NUM_SERVERS],
                             pasis_md_hist_matrix_t *matrix,
                             unsigned int     for_node_id);

void
PasisIO_MDHistCacheDumpData (pasisio_md_hist_data_t *data);


static inline void
PasisIO_MDHistCacheInvalidateData (pasisio_md_hist_data_t *data)
{
#if 0
    data->ts_num = 0;
    memset (data->ts_bitmap, 0, PASIS_MD_HIST_MAX_VERSIONS * sizeof (p_uint64));
    memset (data->auth_num, 0, PASIS_MAX_NUM_SERVERS * sizeof(unsigned int));
#endif
    memset (data, 0, sizeof (pasisio_md_hist_data_t));
}

#endif /* __PASISIO_MD_HIST_CACHE_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */  
