/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASISIO_SELECT_H__
#define __PASISIO_SELECT_H__

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasis_error.h"

#include "pasisio.h"
#include "pasisio_drives.h"
#include "pasisio_config_parse.h"
#include "pasis_md_prot.h"

#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif

/** Drive selection mode */
typedef enum pasisio_select_mode_e {
    PASISIO_SELECT_RANDOM = 0x00,   /* Random */
    PASISIO_SELECT_RNDRBN = 0x01,   /* Round robin */
}  pasisio_select_mode_t;

/** Drive selection flags */
typedef enum pasisio_select_flags_e {
    PASISIO_SELECT_IO_IGNORE = 0x0,
    PASISIO_SELECT_IO_READY = 0x1,    
    PASISIO_SELECT_IO_RETRY = 0x2,
    PASISIO_SELECT_IO_ISSUE = 0x3,
    PASISIO_SELECT_IO_PENDING = 0x4,
    PASISIO_SELECT_IO_SUCCESS = 0x5,
    PASISIO_SELECT_IO_SERVERR = 0x6,
    PASISIO_SELECT_IO_OPERR = 0x7,
    PASISIO_SELECT_IO_TERM = 0x8,
} pasisio_select_flags_t;

typedef struct pasisio_select_s {
    unsigned int               count;
    pasisio_drive_instance_t **instances;
    pasisio_select_flags_t     flags[PASIS_MAX_NUM_SERVERS];
    unsigned int               shuffle;
} pasisio_select_t;

/* FO: I'll use following flag to switch between new and old behavior
 * of selecting quorums; the new quorum selection procedure is in
 * PasisIO_SelectServers(); */
/*#define PASISIO_SELECT_QUORUM */

/** Quorum selection mode - FO **/
typedef enum pasisio_select_quorum_e {
    PASISIO_SELECT_QUORUM_TEST = 0x0, /* access a fixed quorum */
    PASISIO_SELECT_QUORUM_CLOSEST = 0x1, /* access closest */
} pasisio_select_quorum_t;

typedef struct pasisio_ip_dly_s {
    p_uint32 ip_address;
    double dly;
} pasisio_ip_dly_t;

/* End FO */

/* Used by PASIS/2PC/testing program to shuffle order of storage nodes */
#define MDOBJ_SHUFFLE(scheme, block_num) (block_num % (scheme)->n)


int
PasisIO_ReselectServers (unsigned int select_num,
                         int              *select_mask,
                         pasisio_select_t *select_list,
                         pasisio_select_mode_t type,
                         unsigned int instance_type);

int
PasisIO_SelectNewServers (unsigned int select_num,
                          pasisio_select_t *out_list,
                          pasisio_select_t *hint_list,
                          pasisio_select_mode_t type,
                          unsigned int instance_type);

int
PasisIO_SelectServers (int num, pasisio_select_t *instance_list);
                      

void
PasisIO_SelectDecodeLinkages (pasisio_linkage_t *linkages,
                              unsigned int linkage_count,
                              unsigned int type_index,
                              pasisio_select_t *list,
                              int shuffle);

void
PasisIO_SelectThresholdDecodeLinkages (pasisio_linkage_t *linkages,
                                       unsigned int linkage_count, 
                                       unsigned int threshold_count, 
                                       unsigned int type_index,
                                       pasisio_select_t *list,
                                       int shuffle);

void
PasisIO_SelectTreeDecodeLinkages (pasisio_linkage_t *linkages,
                                  pasis_md_protocol_t *protocol,
                                  unsigned int type_index,
                                  pasisio_select_t *list,
                                  int shuffle);

unsigned int
PasisIO_SelectEncodeLinkages (pasisio_linkage_t *linkages,
                              unsigned int linkage_count,
                              pasisio_select_t *list);


#ifdef __cplusplus
}
#endif

#endif /* __PASISIO_SELECT_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */ 
