/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_DIR_OPS_H__
#define __PASIS_MD_DIR_OPS_H__

#include "pasis_config.h"
#include "pasis_types.h"

#include "pasis_md_dir.h"
#include "pasis_md_prot.h"
#include "pasis_md_attr.h"
#include "pasis_md_ops.h"


class MDVersionedObject;
class MDVersion;
class MDObjectID;


static inline void
pasis_md_dir_init_hdr (p_byte *blk)
{
    pasis_md_dir_hdr_t *hdr = (pasis_md_dir_hdr_t *)blk;
    hdr->free_offset = PASIS_MD_DIR_HDR_SIZE;
}


pasis_error_t pasis_md_dir_lookup (p_byte *blk, char *name,
                                   unsigned int *offset_out);

pasis_error_t pasis_md_dir_insert (p_byte *blk, MDObjectID* file,
                                   const char *name, p_uint32 fileid, 
                                   unsigned int *offset_out);

unsigned int pasis_md_dir_remove (p_byte *blk, unsigned int entry_offset);

pasis_error_t pasis_md_dir_create (MDVersionedObject  *parent_dir,
                                   pasis_md_attr_t    *attr_in,
                                   MDVersionedObject  *create_dir,
                                   pasis_md_hts_t     *new_hts,
                                   pasis_md_hts_t     *cond_hts);

pasis_error_t pasis_md_dir_blk_is_empty (MDVersionedObject    *dir,
                                         MDVersion            *version,
                                         pasis_md_attr_t      *attr);

pasis_error_t pasis_md_dir_find_entry (MDVersionedObject    *dir,
                                       MDVersion            &version,
                                       char                 *name,
                                       p_byte              **blk_out,
                                       p_uint64             *blkno_out,
                                       unsigned int         *offset_out,
                                       pasis_md_attr_t      *dir_attr_out);

pasis_error_t pasis_md_dir_rename_parent (MDVersionedObject *dir,
                                          MDVersion         &new_version,
                                          MDVersion         &cond_version,
                                          pasis_md_fid_t    *parent_fid,
                                          p_uint32           parent_fileid);


#endif /* __PASIS_MD_DIR_OPS_H__ */
