/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_HIST_MATRIX_H__
#define __PASIS_MD_HIST_MATRIX_H__

#include "pasis_config.h"
#include "pasis_types.h"

#include "pasis_md_prot.h"


#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif



void 
pasis_md_hist_matrix_init (pasis_md_hist_matrix_t *matrix);

void 
pasis_md_hist_matrix_add_vec (pasis_md_hist_vector_t *vector,
                              pasis_md_hist_matrix_t *matrix);

/* I commented this out, as it's difficult to set the history matrix's
 * auth_indices variables just from the arguments, and the function isn't
 * used. -mabdelmalek */
#if 0
/* auths holds a NxN matrix of authenticators */
void
pasis_md_hist_matrix_set_auths (p_uint32                for_node_id,
                                unsigned int            auth_num,
                                pasis_md_hist_matrix_t *matrix,
                                pasis_md_auth_t        *auths[][]);

/* auths_out holds a NxN matrix of authenticators */
void
pasis_md_hist_matrix_add_auths (pasis_md_hist_vector_t *vec,
                                pasis_md_auth_t        *auths_out[][]);

#endif


void
pasis_md_hist_matrix_copy (pasis_md_hist_matrix_t *dest,
                           pasis_md_hist_matrix_t *src);

void
pasis_md_hist_matrix_get_vec (p_uint32 node_id,
                              pasis_md_hist_matrix_t *matrix,
                              unsigned int *ts_num_out,
                              pasis_md_auth_t **auth_out,
                              pasis_md_hts_t  **recs_out);


/* Returns 1 if the node has an entry in the history matrix contains node for
 * the specified timestamp, 0 otherwise. */
p_uint32
pasis_md_hist_matrix_node_has_ts (p_uint32                node_id,
                                  pasis_md_hist_matrix_t *matrix,
                                  pasis_md_hts_t         *ts);


pasis_error_t
pasis_md_hist_matrix_classify_client (pasis_md_hist_matrix_t *matrix,
                                      pasis_md_protocol_t    *protocol,
                                      pasis_md_hts_t        **ts_classify_out,
                                      pasis_md_hts_t        **ts_latest_out);


void
pasis_md_hist_matrix_classify_node (pasis_md_hist_matrix_t *matrix,
                                    pasis_md_protocol_t    *protocol,
                                    pasis_md_hts_t        **ts_barrier_out,
                                    pasis_error_t          *barrier_status_out,
                                    pasis_md_hts_t        **ts_latest_out,
                                    pasis_error_t          *latest_status_out,
                                    int             *latest_is_multiobject_op,
                                    pasis_md_hts_t        **ts_classify_out,
                                    pasis_error_t          *classify_status_out,
                                    int             *classify_is_multiobject_op,
                                    pasis_md_hts_t        **ts_complete_out);



void
pasis_md_hist_matrix_dump (pasis_md_hist_matrix_t *matrix);



#ifdef __cplusplus
}
#endif


#endif /* __PASIS_MD_HIST_MATRIX_H__ */
