/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_OBJECT_SYNC_H__
#define __PASIS_MD_OBJECT_SYNC_H__

#include "pasis_config.h"
#include "pasis_error.h"

#include "pasis_md_s4.h"
#include "pasis_md_prot.h"



class MDVersionedObject;

/* Functions for performing directory syncing.  This is used during repair. */

void pasis_md_object_sync_stats_init (void);
void pasis_md_object_sync_stats_dump (int shutdown_flag);

pasis_error_t pasis_md_object_sync_init (pasis_md_loc_t  *_distribution,
                                         p_uint32         _node_id);
void pasis_md_object_sync_shutdown (void);


/* new_hts can be null, in which case when we insert a new history entry, we'll
 * use sync_ts as the timestamp.
 * alternatively, for slow repair case in which we object sync an object and
 * want to add it at a newer version (past the barrier), the caller can specify
 * the new timestamp (new_hts). */
pasis_error_t
pasis_md_object_sync_client (p_uint32                  node_id,
                             MDVersionedObject        *object,
                             pasis_md_hts_t           *sync_ts,
                             pasis_md_hts_t           *lcwts_hts,
                             pasis_md_hts_t           *new_hts,
                             pasis_md_hist_matrix_t   *hist_matrix);

void
pasis_md_object_sync_server (pasis_fe_s4_ipc_t        *ipc_op,
                             pasis_md_read_op_args_t  *args,
                             pasis_md_read_op_res_t   *res);



#endif /* __PASIS_MD_OBJECT_SYNC_H__ */
