/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_VALIDATION_H__
#define __PASIS_MD_VALIDATION_H__

#include "pasis_config.h"
#include "pasis_error.h"
#include "pasis_types.h"

#include "pasis_md_prot.h"
#include "pasis_md_ops.h"


#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif


class MDVersionedObject;


void pasis_md_validation_stats_init (void);
void pasis_md_validation_stats_dump (void);


/* Perform validation on the history matrix.  This consists of:
 * 1) Perform read classification (i.e. find the latest complete write
 *    timestamp).  Return an error if no complete write exists.  Otherwise,
 *    lcwts will be set to the latest complete write timestamp (callers will
 *    then condition their operation based on this timestamp).  Also, new_ts
 *    will be set to the latest (possibly incomplete) timestamp + 1.
 * 2) Look up the history vector associated with "fid".
 * 3) Compare the local history vector with the S-N's history vector in the
 *    history matrix.  Return an error on mismatch, to prevent operations using
 *    a stale history matrix. 
 * 4) Verify the authenticators in the history matrix.  Return an error if not
 *    enough of the authenticators of the S-N's containing the latest write are
 *    correct.  (E.g., need Qc+B matching authenticators, just like in read
 *    classification, we need Qc+B matching timestamps.)
 */

pasis_error_t
pasis_md_validate_hist_matrix (p_uint32                node_id,
                               MDVersionedObject      *object,
                               pasis_md_protocol_t    *protocol,
                               pasis_md_hist_matrix_t *hist_matrix,
                               p_uint32                proof_fid_index,
                               pasis_md_hist_matrix_t *proof_matrix,
                               p_byte                 *op_hash, 
                               unsigned int            writing_barrier,
                               pasis_md_hts_t       **cond_ts,   /* Out args */
                               pasis_md_hts_t       **latest_hts, /* Out args */
                               pasis_md_hts_t       **candidate_hts, /* Out*/
                               int                   *already_executed, /* Out*/
                               int                   *doing_repair,    /* Out*/
                               int                   *doing_fast_repair);/* Out*/



#ifdef __cplusplus
}
#endif


#endif /*  __PASIS_MD_VALIDATION_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
