/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#include <math.h>

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasis_error.h"
#include "pasis_debug.h"
#include "pasis_timer.h"

#include "pasis_md_prot.h"
#include "pasis_md_delay.h"
#include "pasis_md_log.h"



/* This is used when calculating hashes over an array of _pointers_ to history
 * entries.  The hash functions need a list of sizes; an entry per each
 * pointer.  Since these are constant, we pre-calculate them. */
double ns_to_iterations;


void
do_loop (unsigned int num_iters)
{
    int i;
    for (i = 0; i < num_iters; ++i) {
        double d1, d2, d3;
        d1 = i * i;
        d2 = num_iters * i;
        d3 = cos (atan (sin (hypot (d1, d2))));
    }
}


void
pasis_md_delay (u_int32_t ms)
{
    do_loop (ns_to_iterations * ms * 1000000);
}


pasis_error_t
pasis_md_delay_init (void)
{
    pasis_timer_t     timer;
    double            num_iters;
    pasis_timespec_t  one_s;

    memset (&one_s, 0, sizeof (pasis_timespec_t));

#if defined(__APPLE__)
    /* Calibrate; this takes approximately 1 second on a G4 550 MHz */
    num_iters = 300000;
#else
    /* Calibrate; this takes approximately 1 second on a P4 2.66 GHz */
    num_iters = 3000000;
#endif
    PASIS_TIMER_START_TM (&timer);
    do_loop (num_iters);
    PASIS_TIMER_STOP_TM (&timer);
    PASIS_TIMER_ELAPSED_TS (&timer, &one_s);

    ns_to_iterations = num_iters / (one_s.ts_sec * 1.0e9 + one_s.ts_nsec);

    return PASIS_SUCCESS;
}


void
pasis_md_delay_shutdown (void)
{
}


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */ 
