/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasis_debug.h"

#include "pasis_md_xdr.h"
#include "pasis_md_prot.h"

#include "rpc_common.h"
#include "pasis_md_svc.h"


int
pasis_md_svc_read_op_reply (rpc_handle_t *server,
                            rpc_req_t *req,
                            pasis_md_read_op_res_t *res)
{
    rpc_tcp_reply_hdr_t reply_hdr;
    int rc;
    //int i;

    req->reply_iov[0].iov_base = (char *)&reply_hdr;
    req->reply_iov[0].iov_len = sizeof (rpc_tcp_reply_hdr_t);
    req->reply_iov_len = 1;

    /* Reply arguments */
    req->reply_iov[1].iov_base = req->rpc_buffer;
    req->reply_iov_cur = 1;

    pasis_md_xdr_enc_read_op_res (req, (u_int32_t *)req->rpc_buffer, res);

    /* RPC reply header */
    rpc_encode_tcp_reply(server, req, &reply_hdr);
    
    /*
    printf ("Read reply transmitting:\n");
    for (i = 0; i < req->reply_iov_len; i++) {
        printf ("- base=%p, len=%u\n", req->reply_iov[i].iov_base,
                req->reply_iov[i].iov_len);
    }
    */

    rc = rpc_transmit_tcp_reply (server, req);

    rpc_req_put (server, req);

    return rc;
}



int
pasis_md_svc_write_op_reply (rpc_handle_t *server,
                             rpc_req_t *req,
                             pasis_md_write_op_res_t *res)
{
    rpc_tcp_reply_hdr_t reply_hdr;
    int rc;
    //int i;

    req->reply_iov[0].iov_base = (char *)&reply_hdr;
    req->reply_iov[0].iov_len = sizeof (rpc_tcp_reply_hdr_t);
    req->reply_iov_len = 1;

    /* Reply arguments */
    req->reply_iov[1].iov_base = (char *)req->rpc_buffer;
    req->reply_iov_cur = 1;

    pasis_md_xdr_enc_write_op_res (req, (u_int32_t *)req->rpc_buffer, res);

    /* RPC reply header */
    rpc_encode_tcp_reply(server, req, &reply_hdr);
    
    /*
    printf ("Read reply transmitting:\n");
    for (i = 0; i < req->reply_iov_len; i++) {
        printf ("- base=%p, len=%u\n", req->reply_iov[i].iov_base,
                req->reply_iov[i].iov_len);
    }
    */

    rc = rpc_transmit_tcp_reply (server, req);

    rpc_req_put (server, req);

    return rc;
}


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
