#!/usr/bin/perl

# The PASIS Query/Update Project
# Copyright (c) 2007, Carnegie Mellon University.
# All rights reserved.
# http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
#
# This software is being provided by the copyright holders under the
# following license. By obtaining, using and/or copying this software,
# you agree that you have read, understood, and will comply with the
# following terms and conditions:
#
# Permission to reproduce, use, and prepare derivative works of this
# software is granted provided the copyright and "No Warranty" statements
# are included with all reproductions and derivative works and associated
# documentation. This software may also be redistributed without charge
# provided that the copyright and "No Warranty" statements are included
# in all redistributions.
#
# NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
# CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
# EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
# TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
# OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
# MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
# TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
# COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
# OR DOCUMENTATION.

use IO::Socket;
use Cwd;
sub logmsg {
    my ( @program_path ) = split (/\//, $0);
    my ( $program_name ) = $program_path[$#program_path];

    my ($Second, $Minute, $Hour, $Day, $Month, $Year, 
        $WeekDay, $DayOfYear, $IsDST) = localtime(time);

    if ($Second < 10) {$Second = "0".$Second;}
    if ($Minute < 10) {$Minute = "0".$Minute;}
    if ($Hour < 10) {$Hour = "0".$Hour;}

    print "$program_name: @_ at [", $Hour, ":", $Minute, ":", $Second, " -- ",
    $Month + 1, "/", $Day, "/", $Year + 1900, "]\n";   
}

my $port = 45678;

`killall pasis_nfs_server 2> /dev/null`;
`killall pasis_md_fe 2> /dev/null`;
`killall pasisio_md_bench 2> /dev/null`;

$ps_out = `ps auxw | grep -v grep | grep pasis`;
print $ps_out unless ($?);

my $sock = new IO::Socket::INET (
                                 LocalPort => $port,
                                 Proto => 'tcp',
                                 Reuse => 1,
                                 Listen => 1,
                                 );

die "Could not create socket: $! ($@)\n" unless $sock;


my %pidhash; 

#get hostname and chop off any .pdl.cmu.edu that might be on the end
my $hostname = `hostname`;
chomp $hostname;
($hostname) = split(/\./,$hostname);


logmsg "Read for connections";


for ( ; $client = $sock->accept; $client->close ()) { 
    $client->autoflush (1);
    logmsg "Connection from ", $client->peerhost, ":", $client->peerport;
    
    #set the working directory before each command is issued
#    $base_path = <$client>;
#    chomp $base_path;
#    chdir $base_path or die "chdir failed for $base_path: $!";;
#    logmsg "Changed working directory to $base_path";
    
    #handle the command
    $proc = <$client>;
    print "-------------------------------NEW RUN\n";
    if ($proc =~ /EXPERIMENT/) {

        # Get experiment parameters
        my $dir = <$client>;
        chomp $dir;
        
        my $cfg_lines = <$client>;
        chomp $cfg_lines;

        # Read in md.cfg file
        my $md_cfg_file = "";
        for (my $i = 0; $i < $cfg_lines; $i++) {
            $md_cfg_file .= <$client>;
        }

        # Create pathname for results
        if (create_dirpath ($dir) == -1) {
            $client->print ("Create dirpath failed: $dir\n");
        }

        # Remove trailing /
        $dir =~ s%/$%%o;

        # Create md.cfg file
        $md_cfg = "$dir/${hostname}_md.cfg";
        logmsg "Experiment: Creating md.cfg file $md_cfg)";
        unless (open (MD_FILE, ">$md_cfg")) {
            logmsg "Creating md.cfg ($md_cfg) failed: $!";
            $client->print ("Create $md_cfg failed");
            next;
        }
        print MD_FILE $md_cfg_file;
        close MD_FILE;
        
        logmsg "created md.cfg file";

        my $exp_type = <$client>;
        chomp $exp_type;

        logmsg "$exp_type";

        # Client experiment
        if ($exp_type =~ /CLIENT/) {

            my $client_id = <$client>;
            chomp $client_id;

            my $num_clients = <$client>;
            chomp $num_clients;

            my $num_processes = <$client>;
            chomp $num_processes;

            my $md_fs = <$client>;
            chomp $md_fs;

            my $mount_point = <$client>;
            chomp $mount_point;

            my $client_format_cmd = <$client>;
            chomp $client_format_cmd;

            my $client_cmd = <$client>;
            chomp $client_cmd;

            `killall -9 pasisio_md_bench 2> /dev/null`;

            if (check_bin ($client_cmd, $client) == -1) {
                next;
            }

            $client_cmd =~ s/HOSTNAME/$hostname/g;
            $client_cmd =~ s/CLIENTID/$client_id/g;


            if ($client_format_cmd) {
                if (check_bin ($client_format_cmd, $client) == -1) {
                    next;
                }

                $client_format_cmd =~ s/HOSTNAME/$hostname/g;
                $client_format_cmd =~ s/CLIENTID/$client_id/g;
            }

            # $md_fs is optional.  If specified, we'll start the NFS server and
            # mount it.  Otherwise, we'll just execute the client command.
            if ($md_fs) {
                if (check_bin ($md_fs, $client) == -1) {
                    next;
                }

                `grep $mount_point /proc/mounts`;
                if ($? == 0) {
                    logmsg "Client: $mount_point already mounted";
                    do {
                        `umount $mount_point`;
                    } while ($? == 0);
                    `grep $mount_point /proc/mounts`;
                    if ($? == 0) {
                        $client->print ("Error: client $mount_point mounted");
                        die "Error: Can't unmount $mount_point\n";
                    }
                }

                unless (-e $mount_point) {
                    $client->print ("Mount point: $mount_point does not exist");
                    logmsg "Mount point: $mount_point does not exist";
                    next;
                }

                $md_fs_out = $dir . "/" . $hostname . "_md_fs.out";
                $md_fs_cmd = "$md_fs -r $md_cfg";
                $md_fs_cmd =~ s/CLIENTID/$client_id/g;
            }


            # Wait for front-end to fully start
            sleep (2); 


            @pid_list=();

            # Do initialization 
            if ($client_format_cmd) {
                for ($i = 0; $i < $num_processes; $i++) {
                    $cmd = $client_format_cmd;
                    $cmd =~ s/PROCESS/$i/g;
                    
                    logmsg "Executing formatting command: $cmd";
                    $out_file = $dir . "/" . $hostname . "_" . $i . "_format_cmd.out";

                    # if (($pid = execute ("$cmd 2>&1 | tee -i $out_file")) == -1){
                    if (($pid = execute ("$cmd > $out_file")) == -1){
                        logmsg "Client format command $cmd failed";
                        $client->print( "Experiment client format $cmd failed");
                        next;
                    }
                    
                    push @pid_list, $pid;
                }
                for ($i = 0; $i < $num_processes; $i++) {
                    waitpid ($pid_list[$i], 0);
                    logmsg "Client -- format command $client_cmd proccess $i done ($?)";
                }
            }

            $client->print("CLIENT INITIALIZATION_DONE\n");
            logmsg "CLIENT INITIALIZATION DONE";

            # Wait for client "START" message
            logmsg "Client waiting for start message";
            $start = <$client>;

            # Run/mount NFS server
            if ($md_fs) {
                logmsg "Client executing $md_fs_cmd";
                if (($md_fs_pid = execute ("$md_fs_cmd > $md_fs_out")) == -1){
                    $client->print("Client execute failed to fork for $md_fs_cmd");
                    logmsg "Client execute failed to fork for $md_fs_cmd";
                    next;
                }

                sleep (2);

                $ps_info = `ps --no-heading -o pid,cmd -C pasis_nfs_server`;
                unless ($ps_info =~ /pasis_nfs_server/) {
                    logmsg "Client error: pasis_nfs_server failed to start";
                    $client->print ("Client failed to start pasis_nfs_server");
                    $output = `cat $md_fs_out`;
                    print $output,"\n";
                }
            
                logmsg "Sleeping", $client_id*2, 
                "seconds, then mounting /pasisfs ";

                sleep ($client_id);

                logmsg "Mounting $mount_point";
                `mount $mount_point`;
                if ($?) {
                    logmsg "Client mount failed on $mount_point";
                    $client->print ("Client mount failed on $mount_point");
                    next;
                }

                logmsg "Creating /pasisfs/$hostname";
                mkdir "/pasisfs/$hostname";

                for ($i = 0; $i < $num_processes; $i++) {
                    mkdir "/pasisfs/${hostname}/${hostname}_${i}";
                }

                logmsg "Sleeping ", $num_clients - $client_id,
                " to synchronize with other clients\n";

                # Synchronize with other clients
                sleep ($num_clients - $client_id);
            }

            @pid_list=();

            # Execute the client command
            for ($i = 0; $i < $num_processes; $i++) {
                $cmd = $client_cmd;
                $cmd =~ s/PROCESS/$i/g;
                
                logmsg "Executing $cmd";
                $out_file = $dir . "/" . $hostname . "_" . $i . "_cmd.out";

                # if (($pid = execute ("$cmd 2>&1 | tee -i $out_file")) == -1){
                if (($pid = execute ("$cmd > $out_file")) == -1){
                    logmsg "Client command $cmd failed";
                    $client->print( "Experiment client $cmd failed");
                    next;
                }
        
                push @pid_list, $pid;
            }

            for ($i = 0; $i < $num_processes; $i++) {
                waitpid ($pid_list[$i], 0);
                logmsg "Client -- $client_cmd proccess $i done ($?)";

                if ($? == 0) {
                    $client->print("EXPERIMENT CLIENT DONE");
                    logmsg "EXPERIMENT CLIENT DONE";
                } else {
                    $client->print("Experiment client error: status $?\n");
                    logmsg "Experiment client error: status $?";
                }
            }

            if ($md_fs) {
                $ps_info = `ps --no-heading -o pid,cmd -C pasis_nfs_server`;
                unless ($ps_info =~ /pasis_nfs_server/) {
                    logmsg "CRASH: pasis_nfs_server not running";
                }

                logmsg "Unmounting $mount_point";

                do {
                    `sudo umount -f $mount_point`;
                } while ($? == 0);
#           `umount $mount_point`;
                `grep $mount_point /proc/mounts`;
                if ($? == 0) {
                    logmsg "Error: Unmounting failed -- exiting";
                    exit (1);
                } else {
                    logmsg "Unmounting succeeded";
                }

                
                logmsg "Shutting down md_fs (pid=$md_fs_pid)";

                kill 'INT', -$md_fs_pid;
                waitpid ($md_fs_pid, 0);

                $ps_info = `ps --no-heading -o pid,cmd -C pasis_nfs_server`;
                if ($ps_info =~ /pasis_nfs_server/) {
                    logmsg "MD_FS stop -- found fs still running, killing all";
                    `killall -INT pasis_nfs_server 2> /dev/null`;
                }
            }

        } elsif ($exp_type =~ /SERVER/) {
            my $node_id = <$client>;
            chomp $node_id;
            
            my $md_fe = <$client>;
            chomp $md_fe;

            if (check_bin ($md_fe, $client) == -1) {
                next;
            }
            
            $out_file = $dir . "/" . $hostname . "_md_fe.out";
            # $server_cmd = "$md_fe -n $node_id -c $md_cfg 2>&1 | tee -i $out_file";
            $server_cmd = "$md_fe -n $node_id -c $md_cfg > $out_file";
                
            logmsg "Server -- command: $server_cmd";
            if (($command_pid = execute ($server_cmd)) == -1){
                $client->print("Server execute failed for $server_cmd");
                logmsg "Server execute failed for $server_cmd";
                next;
            }
        
            $ps_info = `ps --no-heading -o pid,cmd -C pasis_md_fe`;
            logmsg "Server -- command: $ps_info";

            logmsg "Server -- waiting for stop command";

            $stop = <$client>;

            $ps_info = `ps --no-heading -o pid,cmd -C pasis_md_fe`;
            unless ($ps_info =~ /pasis_md_fe/) {
                logmsg "CRASH: pasis_md_fe not running";
            }

            logmsg "Server -- killing command: killall -INT $command_pid";
            kill 'INT', -$command_pid;

            waitpid ($command_pid, 0);

            logmsg "Server -- done, status=$?";

            if ($? == 0) {
                $client->print("EXPERIMENT SERVER DONE");
                logmsg "EXPERIMENT SERVER DONE";
            } else {
                $client->print("Experiment server error: status $?\n");
                logmsg "Experiment server error: status $?";
            }

        } else {
            logmsg "Unknown experiment type: $exp_type";
            next;
        }
    }
    
    #for each pid we generated, see if there is a core file
#     foreach $element (keys %pidhash){
#         if (-e "core.$element"){
            
#             #see how many already exist so we can append a number to it
#             my @nextfree = glob($hostname . "_" . $pidhash{$element} . "*.core");	

#             #rename the core file to something more useful
#             rename "core." . $element, $hostname . "_" . $pidhash{$element} . "_" . scalar @nextfree . ".core";
            
#             die "Warning. Core file found for program $pidhash{$element}\n";
#         }
#     }
    
    logmsg "Connection closed";
    undef %pidhash;
}


sub execute {
    my $pid;
    (my $executable) = @_;
    $pid = fork ();
    if (defined $pid) {
        if ($pid == 0) {
            setpgrp (0, $$);
            exec ($executable);
            
            logmsg "Server -- exec ".$executable." failed";
            
            exit;
        } else {
            #store pid->executable name mappings so we can rename core files
            ($executable) = split(/ /,$executable);
            $executable = substr($executable,2);
            $pidhash{$pid+1} = $executable;
            return $pid;
        }
    } else {
        logmsg "Server -- fork failed ($pid)";
        return -1;
    }

}

sub exec_wait {

    my $pid = execute (@_);
    
    if ($pid) {
        waitpid ($pid, 0);
        
        return $?;
    }

    return -1;
}

sub create_dirpath {
    my ($path) = @_;
    my @paths = split (/\//, $path);
    my $full_path="";

    foreach $path (@paths) {
        if ($path) {
            $full_path .= "/" . $path;

            unless (-d $full_path) {
                my $ret = mkdir $full_path, 0777;
                if ($ret == 0) {
                    logmsg "Creating results dir ($full_path) failed ($!)";
                    return -1;
                }
            }
        }
    }

    return 0;
}

sub check_bin {
    my ($cmd, $handle) = @_;
    
    my ($base_cmd,) = split (/ /, $cmd);
    
    if ((! -B "$base_cmd") || (! -x "$base_cmd")){
        $handle->print("$base_cmd binary not found or not executable ($base_cmd)\n");
        logmsg "$base_cmd binary not found or not executable ($base_cmd)\n";
        return -1;
    }

    return 0;
}


close (Server);


exit;


